@@ -8,8 +8,6 @@
 import sys, struct
 import layoutrom, buildrom
 
-from python23compat import as_bytes
-
 def subst(data, offset, new):
     return data[:offset] + new + data[offset + len(new):]
 
@@ -39,7 +37,7 @@ def main():
                 finalsize = 256*1024
     if datasize > finalsize:
         print("Error!  ROM doesn't fit (%d > %d)" % (datasize, finalsize))
-        print("   You have to either increate the size (CONFIG_ROM_SIZE)")
+        print("   You have to either increase the size (CONFIG_ROM_SIZE)")
         print("   or turn off some features (such as hardware support not")
         print("   needed) to make it fit.  Trying a more recent gcc version")
         print("   might work too.")
@@ -88,7 +86,7 @@ def main():
 
     # Write final file
     f = open(outfile, 'wb')
-    f.write((as_bytes("\0") * (finalsize - datasize)) + rawdata)
+    f.write((b"\0" * (finalsize - datasize)) + rawdata)
     f.close()
 
 if __name__ == '__main__':
