@@ -7,17 +7,19 @@
 
 #include "biosvar.h" // GET_GLOBAL
 #include "output.h" // warn_internalerror
+#include "std/vga.h" // struct video_param_s
 #include "stdvga.h" // stdvga_find_mode
 #include "string.h" // memcpy_far
-#include "vgabios.h" // video_param_table
+#include "vgabios.h" // SET_VGA
+#include "vgautil.h" // vgafont16
 
 
 /****************************************************************
- * Video mode register definitions
+ * Default color palettes (stored in DAC registers)
  ****************************************************************/
 
-/* Mono */
-static u8 palette0[] VAR16 = {
+// 4 grey palette used for MDA card compatibility
+static u8 pal_mda[] VAR16 = {
   0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00,
   0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00,
   0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a, 0x2a,0x2a,0x2a,
@@ -36,7 +38,8 @@ static u8 palette0[] VAR16 = {
   0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f, 0x3f,0x3f,0x3f
 };
 
-static u8 palette1[] VAR16 = {
+// 16 colors used for CGA mode compatibility
+static u8 pal_cga[] VAR16 = {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a,
   0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x15,0x00, 0x2a,0x2a,0x2a,
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a,
@@ -55,7 +58,8 @@ static u8 palette1[] VAR16 = {
   0x3f,0x15,0x15, 0x3f,0x15,0x3f, 0x3f,0x3f,0x15, 0x3f,0x3f,0x3f
 };
 
-static u8 palette2[] VAR16 = {
+// 64 colors used for EGA mode compatibility
+static u8 pal_ega[] VAR16 = {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a,
   0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x2a,0x00, 0x2a,0x2a,0x2a,
   0x00,0x00,0x15, 0x00,0x00,0x3f, 0x00,0x2a,0x15, 0x00,0x2a,0x3f,
@@ -74,7 +78,8 @@ static u8 palette2[] VAR16 = {
   0x3f,0x15,0x15, 0x3f,0x15,0x3f, 0x3f,0x3f,0x15, 0x3f,0x3f,0x3f
 };
 
-static u8 palette3[] VAR16 = {
+// Default 256 colors used in VGA "packed" mode
+static u8 pal_vga[] VAR16 = {
   0x00,0x00,0x00, 0x00,0x00,0x2a, 0x00,0x2a,0x00, 0x00,0x2a,0x2a,
   0x2a,0x00,0x00, 0x2a,0x00,0x2a, 0x2a,0x15,0x00, 0x2a,0x2a,0x2a,
   0x15,0x15,0x15, 0x15,0x15,0x3f, 0x15,0x3f,0x15, 0x15,0x3f,0x3f,
@@ -144,6 +149,11 @@ static u8 palette3[] VAR16 = {
   0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00, 0x00,0x00,0x00
 };
 
+
+/****************************************************************
+ * Video mode register definitions
+ ****************************************************************/
+
 static u8 sequ_01[] VAR16 = { 0x08, 0x03, 0x00, 0x02 };
 static u8 crtc_01[] VAR16 = {
     0x2d, 0x27, 0x28, 0x90, 0x2b, 0xa0, 0xbf, 0x1f,
@@ -260,10 +270,9 @@ static u8 crtc_6A[] VAR16 = {
 struct stdvga_mode_s {
     u16 mode;
     struct vgamode_s info;
-
-    u8 pelmask;
     u8 *dac;
     u16 dacsize;
+
     u8 *sequ_regs;
     u8 miscreg;
     u8 *crtc_regs;
@@ -272,40 +281,40 @@ struct stdvga_mode_s {
 };
 
 static struct stdvga_mode_s vga_modes[] VAR16 = {
-    //mode { model       tx   ty bpp cw ch  sstart    }
-    // pelm  dac            sequ     misc  crtc     actl     grdc
-    {0x00, { MM_TEXT,    40,  25, 4, 9, 16, SEG_CTEXT }
-     , 0xFF, PAL(palette2), sequ_01, 0x67, crtc_01, actl_01, grdc_01},
-    {0x01, { MM_TEXT,    40,  25, 4, 9, 16, SEG_CTEXT }
-     , 0xFF, PAL(palette2), sequ_01, 0x67, crtc_01, actl_01, grdc_01},
-    {0x02, { MM_TEXT,    80,  25, 4, 9, 16, SEG_CTEXT }
-     , 0xFF, PAL(palette2), sequ_03, 0x67, crtc_03, actl_01, grdc_01},
-    {0x03, { MM_TEXT,    80,  25, 4, 9, 16, SEG_CTEXT }
-     , 0xFF, PAL(palette2), sequ_03, 0x67, crtc_03, actl_01, grdc_01},
-    {0x04, { MM_CGA,    320, 200, 2, 8,  8, SEG_CTEXT }
-     , 0xFF, PAL(palette1), sequ_04, 0x63, crtc_04, actl_04, grdc_04},
-    {0x05, { MM_CGA,    320, 200, 2, 8,  8, SEG_CTEXT }
-     , 0xFF, PAL(palette1), sequ_04, 0x63, crtc_04, actl_04, grdc_04},
-    {0x06, { MM_CGA,    640, 200, 1, 8,  8, SEG_CTEXT }
-     , 0xFF, PAL(palette1), sequ_06, 0x63, crtc_06, actl_06, grdc_06},
-    {0x07, { MM_TEXT,    80,  25, 4, 9, 16, SEG_MTEXT }
-     , 0xFF, PAL(palette0), sequ_03, 0x66, crtc_07, actl_07, grdc_07},
-    {0x0D, { MM_PLANAR, 320, 200, 4, 8,  8, SEG_GRAPH }
-     , 0xFF, PAL(palette1), sequ_0d, 0x63, crtc_0d, actl_0d, grdc_0d},
-    {0x0E, { MM_PLANAR, 640, 200, 4, 8,  8, SEG_GRAPH }
-     , 0xFF, PAL(palette1), sequ_0e, 0x63, crtc_0e, actl_0d, grdc_0d},
-    {0x0F, { MM_PLANAR, 640, 350, 1, 8, 14, SEG_GRAPH }
-     , 0xFF, PAL(palette0), sequ_0e, 0xa3, crtc_0f, actl_0f, grdc_0d},
-    {0x10, { MM_PLANAR, 640, 350, 4, 8, 14, SEG_GRAPH }
-     , 0xFF, PAL(palette2), sequ_0e, 0xa3, crtc_0f, actl_10, grdc_0d},
-    {0x11, { MM_PLANAR, 640, 480, 1, 8, 16, SEG_GRAPH }
-     , 0xFF, PAL(palette2), sequ_0e, 0xe3, crtc_11, actl_11, grdc_0d},
-    {0x12, { MM_PLANAR, 640, 480, 4, 8, 16, SEG_GRAPH }
-     , 0xFF, PAL(palette2), sequ_0e, 0xe3, crtc_11, actl_10, grdc_0d},
-    {0x13, { MM_PACKED, 320, 200, 8, 8,  8, SEG_GRAPH }
-     , 0xFF, PAL(palette3), sequ_13, 0x63, crtc_13, actl_13, grdc_13},
-    {0x6A, { MM_PLANAR, 800, 600, 4, 8, 16, SEG_GRAPH }
-     , 0xFF, PAL(palette2), sequ_0e, 0xe3, crtc_6A, actl_10, grdc_0d},
+    //mode { model       tx   ty bpp cw ch  sstart    }  dac
+    // sequ     misc  crtc     actl     grdc
+    {0x00, { MM_TEXT,    40,  25, 4, 9, 16, SEG_CTEXT }, PAL(pal_ega)
+     , sequ_01, 0x67, crtc_01, actl_01, grdc_01},
+    {0x01, { MM_TEXT,    40,  25, 4, 9, 16, SEG_CTEXT }, PAL(pal_ega)
+     , sequ_01, 0x67, crtc_01, actl_01, grdc_01},
+    {0x02, { MM_TEXT,    80,  25, 4, 9, 16, SEG_CTEXT }, PAL(pal_ega)
+     , sequ_03, 0x67, crtc_03, actl_01, grdc_01},
+    {0x03, { MM_TEXT,    80,  25, 4, 9, 16, SEG_CTEXT }, PAL(pal_ega)
+     , sequ_03, 0x67, crtc_03, actl_01, grdc_01},
+    {0x04, { MM_CGA,    320, 200, 2, 8,  8, SEG_CTEXT }, PAL(pal_cga)
+     , sequ_04, 0x63, crtc_04, actl_04, grdc_04},
+    {0x05, { MM_CGA,    320, 200, 2, 8,  8, SEG_CTEXT }, PAL(pal_cga)
+     , sequ_04, 0x63, crtc_04, actl_04, grdc_04},
+    {0x06, { MM_CGA,    640, 200, 1, 8,  8, SEG_CTEXT }, PAL(pal_cga)
+     , sequ_06, 0x63, crtc_06, actl_06, grdc_06},
+    {0x07, { MM_TEXT,    80,  25, 4, 9, 16, SEG_MTEXT }, PAL(pal_mda)
+     , sequ_03, 0x66, crtc_07, actl_07, grdc_07},
+    {0x0D, { MM_PLANAR, 320, 200, 4, 8,  8, SEG_GRAPH }, PAL(pal_cga)
+     , sequ_0d, 0x63, crtc_0d, actl_0d, grdc_0d},
+    {0x0E, { MM_PLANAR, 640, 200, 4, 8,  8, SEG_GRAPH }, PAL(pal_cga)
+     , sequ_0e, 0x63, crtc_0e, actl_0d, grdc_0d},
+    {0x0F, { MM_PLANAR, 640, 350, 1, 8, 14, SEG_GRAPH }, PAL(pal_mda)
+     , sequ_0e, 0xa3, crtc_0f, actl_0f, grdc_0d},
+    {0x10, { MM_PLANAR, 640, 350, 4, 8, 14, SEG_GRAPH }, PAL(pal_ega)
+     , sequ_0e, 0xa3, crtc_0f, actl_10, grdc_0d},
+    {0x11, { MM_PLANAR, 640, 480, 1, 8, 16, SEG_GRAPH }, PAL(pal_ega)
+     , sequ_0e, 0xe3, crtc_11, actl_11, grdc_0d},
+    {0x12, { MM_PLANAR, 640, 480, 4, 8, 16, SEG_GRAPH }, PAL(pal_ega)
+     , sequ_0e, 0xe3, crtc_11, actl_10, grdc_0d},
+    {0x13, { MM_PACKED, 320, 200, 8, 8,  8, SEG_GRAPH }, PAL(pal_vga)
+     , sequ_13, 0x63, crtc_13, actl_13, grdc_13},
+    {0x6A, { MM_PLANAR, 800, 600, 4, 8, 16, SEG_GRAPH }, PAL(pal_ega)
+     , sequ_0e, 0xe3, crtc_6A, actl_10, grdc_0d},
 };
 
 
@@ -348,9 +357,18 @@ stdvga_list_modes(u16 seg, u16 *dest, u16 *last)
     SET_FARVAR(seg, *dest, 0xffff);
 }
 
+static struct video_save_pointer_s video_save_pointer_table VAR16;
+
+static struct video_param_s video_param_table[29] VAR16;
+
 void
 stdvga_build_video_param(void)
 {
+    SET_BDA(video_savetable
+            , SEGOFF(get_global_seg(), (u32)&video_save_pointer_table));
+    SET_VGA(video_save_pointer_table.videoparam
+            , SEGOFF(get_global_seg(), (u32)video_param_table));
+
     static u8 parammodes[] VAR16 = {
         0, 0, 0, 0, 0x04, 0x05, 0x06, 0x07,
         0, 0, 0, 0, 0, 0x0d, 0x0e, 0,
@@ -401,9 +419,9 @@ stdvga_build_video_param(void)
     // Fill available legacy modes in video_func_static table
     u32 modes = 0;
     for (i = 0; i < ARRAY_SIZE(vga_modes); i++) {
-        u16 mode = vga_modes[i].mode;
+        u16 mode = GET_GLOBAL(vga_modes[i].mode);
         if (mode <= 0x13)
-            modes |= 1<<i;
+            modes |= 1<<mode;
     }
     SET_VGA(static_functionality.modes, modes);
 }
@@ -420,19 +438,17 @@ stdvga_override_crtc(int mode, u8 *crtc)
 }
 
 static void
-clear_screen(struct vgamode_s *vmode_g)
+clear_screen(struct vgamode_s *curmode_g)
 {
-    switch (GET_GLOBAL(vmode_g->memmodel)) {
+    switch (GET_GLOBAL(curmode_g->memmodel)) {
     case MM_TEXT:
-        memset16_far(GET_GLOBAL(vmode_g->sstart), 0, 0x0720, 32*1024);
+        memset16_far(GET_GLOBAL(curmode_g->sstart), 0, 0x0720, 32*1024);
         break;
     case MM_CGA:
-        memset16_far(GET_GLOBAL(vmode_g->sstart), 0, 0x0000, 32*1024);
+        memset16_far(GET_GLOBAL(curmode_g->sstart), 0, 0x0000, 32*1024);
         break;
     default:
-        // XXX - old code gets/sets/restores sequ register 2 to 0xf -
-        // but it should always be 0xf anyway.
-        memset16_far(GET_GLOBAL(vmode_g->sstart), 0, 0x0000, 64*1024);
+        memset16_far(GET_GLOBAL(curmode_g->sstart), 0, 0x0000, 64*1024);
     }
 }
 
@@ -448,18 +464,18 @@ stdvga_set_mode(struct vgamode_s *vmode_g, int flags)
 
     // if palette loading (bit 3 of modeset ctl = 0)
     if (!(flags & MF_NOPALETTE)) {    // Set the PEL mask
-        stdvga_pelmask_write(GET_GLOBAL(stdmode_g->pelmask));
+        stdvga_pelmask_write(0xff);
 
         // From which palette
         u8 *palette_g = GET_GLOBAL(stdmode_g->dac);
         u16 palsize = GET_GLOBAL(stdmode_g->dacsize) / 3;
 
         // Always 256*3 values
-        stdvga_dac_write(get_global_seg(), palette_g, 0, palsize);
+        stdvga_dac_write_many(get_global_seg(), palette_g, 0, palsize);
         int i;
         for (i = palsize; i < 0x0100; i++) {
-            static u8 rgb[3] VAR16;
-            stdvga_dac_write(get_global_seg(), rgb, i, 1);
+            struct vbe_palette_entry rgb = { };
+            stdvga_dac_write(i, rgb);
         }
 
         if (flags & MF_GRAYSUM)
@@ -519,5 +535,5 @@ stdvga_set_mode(struct vgamode_s *vmode_g, int flags)
 void
 stdvga_set_packed_palette(void)
 {
-    stdvga_dac_write(get_global_seg(), palette3, 0, sizeof(palette3) / 3);
+    stdvga_dac_write_many(get_global_seg(), pal_vga, 0, sizeof(pal_vga) / 3);
 }
